/*  AOS Tool, decrypt and decompress Archos Gmini firmware files
    Copyright (C) 2004  Michael R. Donat <michael@donat.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
   To do:
     * replicate bug in original Archos packer for firmware versions
       Gmini120: v1.2 and v1.8,  Gmini220: v1.6, and GminiSP: v1.2
     * determine the use of the contents of the SIGN chunk
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#ifndef _BORLAND_
 #include <sys/ioctl.h>
 #include <unistd.h>
 #include <sys/io.h>
 #define DELIM '/'
#else
 #include <io.h>
 #define DELIM '\\'
#endif
#include "types.h"
#include "util.h"

u8* getText( const u8* src, u32 slen, u8* dest, u32 dlen);
u32 getByteCount( const u8* data, u32 length );
u8* guessXORkey( const u8* source, u32 length, u32 keylength );
u8* XOR( u8* data, u32 length, u32 keylength );
void displayAJZ( const char* type, u8* ajz, u32 csize, u32 ucsize );
void displayChunk( u8 *chunk, u32 leng );
int processAOS( const char* file );

u8* key;
char *filename;

u8* getText( const u8* src, u32 slen, u8* dest, u32 dlen) {
   u8 flag = 0;
   u8 used = 7;
   u32 si = 0;
   u32 di = 0;
   while( di < dlen && si < slen )
   {
      flag >>= 1;
      if( ++used == 8 ) {
         flag = src[si++];
         used = 0;
      }
      if( flag & 0x01 )
         dest[di++]=src[si++];
      else
      {
         u32 offset = (di & 0xfffff000) + 18 +
                      ((u32)src[si] | (((u32)src[si+1] & 0xf0) << 4));
         u8 count = (src[si+1] & 0x0f) + 3;
         si +=2;
         if( offset > di )
            offset -= 4096;
         for(;count > 0; --count )
            dest[di++] = dest[offset++];
      }
   }
   return dest;
}

u32 getByteCount( const u8* data, u32 length ) {
   u8 flag = 0;
   u8 used = 7;
   u32 count = 0;
   const u8* end = data + length;
   while( data < end ) {
      flag >>= 1;
      if( ++used == 8 ) {
         flag = *(data++);
         used = 0;
      }
      if( flag & 0x01 )
         ++count;
      else
         count += (*(++data) & 0x0f) + 3;
      ++data;
   }
   return count;
}

u8* guessXORkey( const u8* source, u32 length, u32 keylength ) {
   key = (u8*)malloc( keylength + 1);
   key[keylength] = '\0';
   u32 i;
   u32 bytecount[256];
   u32 c;
   u8 v;
   for( i = 0; i < keylength; ++i )
   {
      for( c = 0; c < 256; ++c )
         bytecount[c] = '\0';
      /* gather byte frequency statistics */
      for( c = i; c < length; c += keylength )
      {
         v = source[c];
         if( (v >= 'A' && v <= 'Z') || (v >='z' && v <='z') )
         ++bytecount[v];
      }
      u32 topval = 0;
      u8 topchar = 0;
      for( c = 0; c < 256; ++c )
         if( bytecount[c] > topval )
         {
            topval = bytecount[c];
            topchar = c;
         }
      key[i] = topchar;
   }
   return key;
}

u8* XOR( u8* data, u32 length, u32 keylength ) {
   u32 i;
   for( i = 0; i < length; ++i )
      data[i] ^= key[ i % keylength ];
   return data;
}

void displayAJZ( const char* type, u8* ajz, u32 csize, u32 ucsize ) {
   // AJZ header ID field
   char head[5];
   u8* image = ajz + 16;
   u32 size;
   cstr( head, ajz, 4 );
   printf( "%s: SIGNATURE: %s\n", type, head );

   // Possibly buggy compressed size stored in AJZ header
   size = lu32( ajz + 8 );
   if ( size != csize )
      printf( "%s: BUG in Archos packer: compressed size: %u\n", type, size );
   printf( "%s: Compressed size of firmware: %u\n", type, csize );
   // Checksum stored in AJZ header
   printf( "%s: Checksum: %08X\n", type, lu32( ajz + 12) );

   // Checksum of AJZ payload
   u32 sumdata = chksum32( ajz + 16, csize );
   if( lu32( ajz + 12) != sumdata )
      printf( "%s: Checksum of data (%08X) doesn't match header\n", type, sumdata );

   // Decrypt firmware Image
   if( key == NULL )
   {
      key = guessXORkey( image, csize, 32 );
      printf( "%s: Guessing key: %s\n", type, key );
   }
   else
      printf( "%s: Using key: %s\n", type, key );
   XOR( image, csize, strlen((char*)key));

   // Decompress firmware Image
   // Possibly buggy uncompressed size stored in AJZ header
   size = lu32( ajz + 4 );
   u32 actual = getByteCount( image, csize );
   if ( size != ucsize )
      printf( "%s: BUG in Archos packer: uncompressed size.: %u\n", type, size );
   if ( size != actual )
      printf( "%s: BUG in Archos packer: uncompressed size..: %u\n", type, size );
   if ( ucsize != actual )
      printf( "%s: BUG in Archos packer: uncompressed size...: %u\n", type, ucsize );
   printf( "%s: Uncompressed Size: %u\n", type, actual );
   u8* firmware = (u8*)malloc( ucsize );
   getText( image, csize, firmware, ucsize );

   // Write unencrypted/decompressed file to disk
   printf( "%s: Writing decompressed firmware to %s\n", type, filename );
   writeFile( filename, firmware, actual );
   printf( "\n" );
   free( firmware );
}

void displayChunk( u8 *chunk, u32 leng ) {
   char type[5];
   u32 cleng;
   u32 ucsize = 0;
   u8 *end = chunk + leng;
   while( chunk + 8 < end ) {
      cstr( type, chunk, 4 );
      cleng = bu32( chunk + 4 );
      if( strcmp( type, "RIFF" ) == 0 || strcmp( type, "LIST" ) == 0 )
         displayChunk( chunk + 8, cleng );
      else if( strcmp( type, "SWID" ) == 0 ) {
         char vid[5]; vid[4] = 0;
         strncpy( vid, (char*)(chunk + 8), 4 );
         printf( "%s: Version ID %s\n", type, vid);
         printf( "%s: Version string: %s\n", type, chunk + 12 );
      }
      else if( strcmp( type, "FLSH" ) == 0 ) {
         ucsize = bu32( chunk + 12 );
         if( bu32( chunk + 8 ) != 24576 )
            printf( "%s: %08X != %08X\n", type, bu32( chunk + 8 ), 24576 );
      }
      else if( strcmp( type, "HWID" ) == 0 )
         {if(bu32(chunk + 8)!= 0)printf( "%s: %u != 0\n", type, bu32( chunk + 8 ) );}
      else if( strcmp( type, "CCOD" ) == 0 )
         displayAJZ( type, chunk + 12, bu32( chunk + 8 ) - 16, ucsize );
      else if( strcmp( type, "CHOS" ) == 0 )
         {if( cleng != 0 ) printf( "%s: length %u != 0\n", type, cleng );}
      else if( strcmp( type, "SIGN" ) == 0 )
         {if( cleng != 108 ) printf( "%s: length %u != 0\n", type, cleng );}
      else
         printf( "%s: length: %u\n", type, cleng );
      chunk += ((cleng==0)? 8: cleng);
   }
}

int processAOS( const char* file ) {
   u8 *aos;
   u32 length;
   if( !( length = loadFile( file, &aos )))
      return -1;
   printf( "File: %s (%u bytes)\n", file, length );
   displayChunk( aos, length );
   free( aos );
   return 1;
}

int main( int argc, char** argv ) {
   int n = 1;
   if( argc < 2 )
      printf("Usage: %s file.aos [file2.aos ...]\n", argv[0] + strrpos( argv[0], DELIM ));
   else for(;n < argc; ++n ) {
      int len = strlen(argv[n]);
      int pre = strrpos( argv[n], DELIM );
      len -= pre;
      filename = (char*)malloc( len );
      strncpy( filename, argv[n] + pre, len - 3 );
      strcpy( filename + len - 3, "bin" );
      processAOS( argv[n] );
      free( filename );
   }
   return 0;
}

