/*  disasmTCC730, decode TCC730 (CalmRISC16 / MAC2424) machine code
         to assembly mnemonics.
    file: formatOP.c, functions to format instruction mnemonics
          for the CalmRISC core of the TCC730

    Copyright (C) 2004  Michael R. Donat <michael@donat.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include "types.h"
#include "formatCOP.h"
#include "formatOP.h"

char OPformat[256];

s32 sext8to32( u32 val ) {
   if( val & 0x00000080 )
      val |= 0xffffff00;
   return (s32)val;
}
s32 sext11to32( u32 val ) {
   if( val & 0x00000400 )
      val |= 0xfffff800;
   return (s32)val;
}
s32 sext13to32( u32 val ) {
   if( val & 0x00001000 )
      val |= 0xffffe000;
   return (s32)val;
}

char *formatOP( int mnnum, u16 op1, u16 op2, u32 ip ) {
   /* operands */
   u8 An, Am, Ai,
      Dn, Di,
      Rn, Rm, Ri, Rj,
      En, Ei,
      bs3, EC, H,
      imm5, imm6, imm7,
      disp4, edisp5;
   u16 imm8, imm13, imm16,
       edisp9, disp16;
   u32 imm22, adr22;
   s32 eoffset8, eoffset11, eoffset13;
   /* temp variables */
   u32 ipr;
   switch( mnnum ) {
      case 0:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         imm7 = (u8)(op1 & (u16)0x007f);
         sprintf( OPformat, "ADD R%u, #%u", Rn , imm7 );
         break;
      case 1:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         imm7 = (u8)(op1 & (u16)0x007f);
         sprintf( OPformat, "SUB R%u, #%u", Rn, imm7 );
         break;
      case 2:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         imm8 = (u8)(op1 & (u16)0x00ff);
         sprintf( OPformat, "LD R%u, #%u", Rn, imm8 );
         break;
      case 3:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         edisp9 = ((op1 & (u16)0x00ff) << 1);
         sprintf( OPformat, "LDW R%u, @[SP + %u]", Rn , edisp9 );
         break;
      case 4:
         Ri = (u8)((op1 & (u16)0x0f00) >> 8);
         edisp9 = ((op1 & (u16)0x00ff) << 1);
         sprintf( OPformat, "LDW @[SP + %u], R%u", edisp9, Ri );
         break;
      case 5:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ai = (u8)(((op1 & (u16)0x0070) >> 4) + 8);
         edisp5 = ((op1 & (u16)0x000f) << 1);
         sprintf( OPformat, "LDW R%u, @[A%u + %u]", Rn, Ai, edisp5 );
         break;
      case 6:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ai = (u8)(((op1 & (u16)0x0070) >> 4) + 8);
         Rj = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "LDW R%u, @[A%u + R%u]", Rn, Ai, Rj );
         break;
      case 7:
         An = (u8)(((op1 & (u16)0x0070) >> 4) + 8);
         edisp5 = ((op1 & (u16)0x000f) << 1);
         Ri = (u8)((op1 & (u16)0x0f00) >> 8);
         sprintf( OPformat, "LDW @[A%u + %d], R%u", An, edisp5, Ri );
         break;
      case 8:
         An = (u8)(((op1 & (u16)0x0070) >> 4) + 8);
         Rm = (u8)((op1 & (u16)0x000f));
         Ri = (u8)((op1 & (u16)0x0f00) >> 8);
         sprintf( OPformat, "LDW @[A%u + R%u], R%u", An, Rm, Ri );
         break;
      case 9:
         Dn = (u8)((op1 & (u16)0x0700) >> 8);
         Ai = (u8)(((op1 & (u16)0x0070) >> 4) + 8);
         disp4 = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "LDB R%u, @[A%u + %u]", Dn, Ai , disp4 );
         break;
      case 10:
         Dn = (u8)((op1 & (u16)0x0700) >> 8);
         Ai = (u8)(((op1 & (u16)0x0070) >> 4) + 8);
         Rj = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "LDB R%u, @[A%u + R%u]", Dn, Ai, Rj );
         break;
      case 11:
         An = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         Ai = (u8)(((op1 & (u16)0x0070) >> 4) + 8);
         disp4 = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "LDW A%u, @[A%u + %u]", An, Ai, disp4 );
         break;
      case 12:
         An = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         Ai = (u8)(((op1 & (u16)0x0070) >> 4) + 8);
         Rj = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "LDW A%u, @[A%u + R%u]", An, Ai, Rj );
         break;
      case 13:
         An = (u8)(((op1 & (u16)0x0070) >> 4) + 8);
         disp4 = (u8)(op1 & (u16)0x000f);
         Di = (u8)((op1 & (u16)0x0700) >> 8);
         sprintf( OPformat, "LDB @[A%u + %u], R%u", An, disp4, Di );
         break;
      case 14:
         An = (u8)(((op1 & (u16)0x0070) >> 4) + 8);
         Rm = (u8)(op1 & (u16)0x000f);
         Di = (u8)((op1 & (u16)0x0700) >> 8);
         sprintf( OPformat, "LDB @[A%u + R%u], R%u", An, Rm, Di );
         break;
      case 15:
         An = (u8)(((op1 & (u16)0x0070) >> 4) + 8);
         disp4 = (u8)(op1 & (u16)0x000f);
         Ai = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         sprintf( OPformat, "LDW @[A%u + %u], A%u", An, disp4, Ai );
         break;
      case 16:
         An = (u8)(((op1 & (u16)0x0070) >> 4) + 8);
         Rm = (u8)(op1 & (u16)0x000f);
         Ai = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         sprintf( OPformat, "LDW @[A%u + R%u], A%u", An, Rm, Ai );
         break;
      case 17:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "ADD R%u, R%u", Rn, Ri );
         break;
      case 18:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "SUB R%u, R%u", Rn, Ri );
         break;
      case 19:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "ADC R%u, R%u", Rn, Ri );
         break;
      case 20:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "SBC R%u, R%u", Rn, Ri );
         break;
      case 21:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "AND R%u, R%u", Rn, Ri );
         break;
      case 22:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "OR R%u, R%u", Rn, Ri );
         break;
      case 23:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "XOR R%u, R%u", Rn, Ri );
         break;
      case 24:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "TST R%u, R%u", Rn, Ri );
         break;
      case 25:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "CMPGE R%u, R%u", Rn, Ri );
         break;
      case 26:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "CMPGT R%u, R%u", Rn, Ri );
         break;
      case 27:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "CMPUGE R%u, R%u", Rn, Ri );
         break;
      case 28:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "CMPUGT R%u, R%u", Rn, Ri );
         break;
      case 29:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "CMPEQ, R%u, R%u", Rn, Ri );
         break;
      case 30:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "LD R%u, R%u", Rn, Ri );
         break;
      case 31:
         Rn = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "RR R%u", Rn );
         break;
      case 32:
         Rn = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "RL R%u", Rn );
         break;
      case 33:
         Rn = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "RRC R%u", Rn );
         break;
      case 34:
         Rn = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "SRB R%u", Rn );
         break;
      case 35:
         Rn = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "SR R%u", Rn );
         break;
      case 36:
         Rn = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "SRA R%u", Rn );
         break;
      case 37:
         Ai = (u8)((op1 & (u16)0x0007) + 8);
         sprintf( OPformat, "JPF A%u", Ai );
         break;
      case 38:
         Ai = (u8)((op1 & (u16)0x0007) + 8);
         sprintf( OPformat, "JPT A%u", Ai );
         break;
      case 39:
         Ai = (u8)((op1 & (u16)0x0007) + 8);
         if( Ai == 14 ) /* A14 == Stack Pointer */
            sprintf( OPformat, "JMP A%u (Return)", Ai );
         else
            sprintf( OPformat, "JMP A%u", Ai );
         break;
      case 40:
         Ai = (u8)((op1 & (u16)0x0007) + 8);
         sprintf( OPformat, "JSR A%u", Ai );
         break;
      case 41:
         Rn = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "SLB R%u", Rn );
         break;
      case 42:
         Rn = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "DT R%u", Rn );
         break;
      case 43:
         Rn = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "INCC R%u", Rn );
         break;
      case 44:
         Rn = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "DECC R%u", Rn );
         break;
      case 45:
         Rn = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "COM R%u", Rn );
         break;
      case 46:
         Rn = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "COM2 R%u", Rn );
         break;
      case 47:
         Rn = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "COMC R%u", Rn );
         break;
      case 48:
         Rn = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "EXT R%u", Rn );
         break;
      case 49:
         Rn = (u8)(op1 & (u16)0x000f);
         imm16 = (u16)op2;
         sprintf( OPformat, "ADD R%u, #%u", Rn, imm16 );
         break;
      case 50:
         An = (u8)((op1 & (u16)0x0007) + 8);
         imm16 = (u16)op2;
         sprintf( OPformat, "ADD A%u, #%04XH", An, imm16 );
         break;
      case 51:
         An = (u8)((op1 & (u16)0x0007) + 8);
         imm16 = (u16)op2;
         sprintf( OPformat, "SUB A%u, #%04XH", An, imm16 );
         break;
      case 52:
         Rn = (u8)(op1 & (u16)0x000f);
         imm16 = (u16)op2;
         sprintf( OPformat, "ADC R%u, #%u", Rn, imm16 );
         break;
      case 53:
         Rn = (u8)(op1 & (u16)0x000f);
         imm16 = (u16)op2;
         sprintf( OPformat, "SBC R%u, #%u", Rn, imm16 );
         break;
      case 54:
         Rn = (u8)(op1 & (u16)0x000f);
         imm16 = (u16)op2;
         sprintf( OPformat, "AND R%u, #%04XH", Rn, imm16 );
         break;
      case 55:
         Rn = (u8)(op1 & (u16)0x000f);
         imm16 = (u16)op2;
         sprintf( OPformat, "OR R%u, #%04XH", Rn, imm16 );
         break;
      case 56:
         Rn = (u8)(op1 & (u16)0x000f);
         imm16 = (u16)op2;
         sprintf( OPformat, "XOR R%u, #%04XH", Rn, imm16 );
         break;
      case 57:
         Rn = (u8)(op1 & (u16)0x000f);
         imm16 = (u16)op2;
         sprintf( OPformat, "TST R%u, #%04XH", Rn, imm16 );
         break;
      case 58:
         Rn = (u8)(op1 & (u16)0x000f);
         imm16 = (u16)op2;
         sprintf( OPformat, "CMPGE R%u, #%u", Rn, imm16 );
         break;
      case 59:
         Rn = (u8)(op1 & (u16)0x000f);
         imm16 = (u16)op2;
         sprintf( OPformat, "CMPGT R%u, #%u", Rn, imm16 );
         break;
      case 60:
         Rn = (u8)(op1 & (u16)0x000f);
         imm16 = (u16)op2;
         sprintf( OPformat, "CMPUGE R%u, #%u", Rn, imm16 );
         break;
      case 61:
         Rn = (u8)(op1 & (u16)0x000f);
         imm16 = (u16)op2;
         sprintf( OPformat, "CMPUGT R%u, #%u", Rn, imm16 );
         break;
      case 62:
         Rn = (u8)(op1 & (u16)0x000f);
         imm16 = (u16)op2;
         sprintf( OPformat, "CMPEQ R%u, #%u", Rn, imm16 );
         break;
      case 63:
         Rn = (u8)(op1 & (u16)0x000f);
         imm16 = (u16)op2;
         sprintf( OPformat, "LD R%u, #%u", Rn, imm16 );
         break;
      case 64:
         sprintf( OPformat, "Reserved" );
         break;
      case 65:
         Dn = (u8)((op1 & (u16)0x0700) >> 8);
         imm8 = (u8)(op1 & (u16)0x00ff);
         sprintf( OPformat, "CMPEQ R%u, #%u", Dn, imm8 );
         break;
      case 66:
         imm8 = (u8)(op1 & (u16)0x00ff);
         sprintf( OPformat, "AND R0, #%02XH", imm8 );
         break;
      case 67:
         imm8 = (u8)(op1 & (u16)0x00ff);
         sprintf( OPformat, "OR R0, #%02XH", imm8 );
         break;
      case 68:
         imm8 = (u8)(op1 & (u16)0x00ff);
         sprintf( OPformat, "XOR R0, #%02XH", imm8 );
         break;
      case 69:
         imm8 = (u8)(op1 & (u16)0x00ff);
         sprintf( OPformat, "TST R0, #%02XH", imm8 );
         break;
      case 70:
         imm8 = (u8)(op1 & (u16)0x00ff);
         sprintf( OPformat, "LDB R0, @[A8 + %u]", imm8 );
         break;
      case 71:
         imm8 = (u8)(op1 & (u16)0x00ff);
         sprintf( OPformat, "LDB @[A8 + %u],R0", imm8 );
         break;
      case 72:
         bs3 = (u8)(op1 & (u16)0x0007);
         sprintf( OPformat, "BITR @[A8 + R1], %u", bs3 );
         break;
      case 73:
         bs3 = (u8)(op1 & (u16)0x0007);
         sprintf( OPformat, "BITS @[A8 + R1], %u", bs3 );
         break;
      case 74:
         bs3 = (u8)(op1 & (u16)0x0007);
         sprintf( OPformat, "BITC @[A8 + R1], %u", bs3 );
         break;
      case 75:
         bs3 = (u8)(op1 & (u16)0x0007);
         sprintf( OPformat, "BITT @[A8 + R1], %u", bs3 );
         break;
      case 76:
         imm5 = (u8)(op1 & (u16)0x001f);
         sprintf( OPformat, "SYS #%u", imm5 );
         break;
      case 77:
         imm6 = (u8)(op1 & (u16)0x003f);
         sprintf( OPformat, "SWI #%u", imm6 );
         break;
      case 78:
         bs3 = (u8)(op1 & (u16)0x0007);
         sprintf( OPformat, "CLRSR %u", bs3 );
         break;
      case 79:
         bs3 = (u8)(op1 & (u16)0x0007);
         sprintf( OPformat, "SETSR %u", bs3 );
         break;
      case 80:
         bs3 = (u8)(op1 & (u16)0x0007);
         sprintf( OPformat, "TSTSR %u", bs3 );
         break;
      case 81:
         sprintf( OPformat, "NOP" );
         break;
      case 82:
         sprintf( OPformat, "BREAK" );
         break;
      case 83:
         sprintf( OPformat, "LD R0, SR" );
         break;
      case 84:
         sprintf( OPformat, "LD SR, R0" );
         break;
      case 85:
         sprintf( OPformat, "RET_FIQ" );
         break;
      case 86:
         sprintf( OPformat, "RET_IRQ" );
         break;
      case 87:
         sprintf( OPformat, "RET_SWI" );
         break;
      case 88:
         sprintf( OPformat, "RETD" );
         break;
      case 89:
         sprintf( OPformat, "LD R0, SPCL_FIQ" );
         break;
      case 90:
         sprintf( OPformat, "LD R0, SPCH_FIQ" );
         break;
      case 91:
         sprintf( OPformat, "LD R0, SSR_FIQ" );
         break;
      case 92:
         sprintf( OPformat, "Reserved" );
         break;
      case 93:
         sprintf( OPformat, "LD R0, SPCL_IRQ" );
         break;
      case 94:
         sprintf( OPformat, "LD R0, SPCH_IRQ" );
         break;
      case 95:
         sprintf( OPformat, "LD R0, SSR_IRQ" );
         break;
      case 96:
         sprintf( OPformat, "Reserved" );
         break;
      case 97:
         sprintf( OPformat, "Reserved" );
         break;
      case 98:
         sprintf( OPformat, "LD R0, SSR_SWI" );
         break;
      case 99:
         sprintf( OPformat, "Reserved" );
         break;
      case 100:
         sprintf( OPformat, "Reserved" );
         break;
      case 101:
         sprintf( OPformat, "LD SPCL_FIQ, R0" );
         break;
      case 102:
         sprintf( OPformat, "LD SPCH_FIQ, R0" );
         break;
      case 103:
         sprintf( OPformat, "LD SSR_FIQ, R0" );
         break;
      case 104:
         sprintf( OPformat, "Reserved" );
         break;
      case 105:
         sprintf( OPformat, "LD SPCL_IRQ, R0" );
         break;
      case 106:
         sprintf( OPformat, "LD SPCH_IRQ, R0" );
         break;
      case 107:
         sprintf( OPformat, "LD SSR_IRQ, R0" );
         break;
      case 108:
         sprintf( OPformat, "Reserved" );
         break;
      case 109:
         sprintf( OPformat, "Reserved" );
         break;
      case 110:
         sprintf( OPformat, "LD SSR_SWI, R0" );
         break;
      case 111:
         sprintf( OPformat, "Reserved" );
         break;
      case 112:
         sprintf( OPformat, "Reserved" );
         break;
      case 113:
         sprintf( OPformat, "Reserved" );
         break;
      case 114:
         sprintf( OPformat, "Reserved" );
         break;
      case 115:
         An = (u8)((op1 & (u16)0x0007) + 8);
         sprintf( OPformat, "LD A%u, PC", An );
         break;
      case 116:
         sprintf( OPformat, "Reserved" );
         break;
      case 117:
         adr22 = (u32)(((op1 & (u16)0x003f) << 16) | op2);
         sprintf( OPformat, "JPF %06XH", adr22 );
         break;
      case 118:
         adr22 = (u32)(((op1 & (u16)0x003f) << 16) | op2);
         sprintf( OPformat, "JPT %06XH", adr22 );
         break;
      case 119:
         adr22 = (u32)(((op1 & (u16)0x003f) << 16) | op2);
         sprintf( OPformat, "JMP %06XH", adr22 );
         break;
      case 120:
         adr22 = (u32)(((op1 & (u16)0x003f) << 16) | op2);
         sprintf( OPformat, "JSR %06XH", adr22 );
         break;
      case 121:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ai = (u8)((op1 & (u16)0x0007) + 8);
         sprintf( OPformat, "LDC R%u, @A%u", Rn, Ai );
         break;
      case 122:
         sprintf( OPformat, "Reserved" );
         break;
      case 123:
         Dn = (u8)((op1 & (u16)0x0700) >> 8);
         Ei = (u8)((op1 & (u16)0x0007) + 8);
         sprintf( OPformat, "LD R%u, E%u", Dn, Ei );
         break;
      case 124:
         En = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         Di = (u8)(op1 & (u16)0x0007);
         sprintf( OPformat, "LD E%u, R%u", En, Di );
         break;
      case 125:
         An = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         Ai = (u8)((op1 & (u16)0x0007) + 8);
         sprintf( OPformat, "CMPEQ A%u, A%u", An, Ai );
         break;
      case 126:
         An = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         Ai = (u8)((op1 & (u16)0x0007) + 8);
         sprintf( OPformat, "LD A%u, A%u", An, Ai );
         break;
      case 127:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Ai = (u8)((op1 & (u16)0x0007) + 8);
         disp16 = (u16)op2;
         sprintf( OPformat, "LDW R%u, @[A%u + %u]", Rn, Ai, disp16 );
         break;
      case 128:
         An = (u8)((op1 & (u16)0x0007) + 8);
         disp16 = (u16)op2;
         Ri = (u8)((op1 & (u16)0x0f00) >> 8);
         sprintf( OPformat, "LDW @[A%u + %u], R%u", An, disp16, Ri );
         break;
      case 129:
         Dn = (u8)((op1 & (u16)0x0700) >> 8);
         Ai = (u8)((op1 & (u16)0x0007) + 8);
         disp16 = (u16)op2;
         sprintf( OPformat, "LDB R%u, @[A%u + %u]", Dn, Ai, disp16 );
         break;
      case 130:
         An = (u8)((u8)((op1 & (u16)0x0007) + 8));
         disp16 = (u16)op2;
         Di = (u8)((u8)((op1 & (u16)0x0700) >> 8));
         sprintf( OPformat, "LDB @[A%u + %u], R%u", An, disp16, Di );
         break;
      case 131:
         An = (u8)((op1 & (u16)0x0007) + 8);
         Ai = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         disp16 = (u16)op2;
         sprintf( OPformat, "LDW A%u, @[A%u + %u]", An, Ai, disp16 );
         break;
      case 132:
         An = (u8)((op1 & (u16)0x0007) + 8);
         disp16 = (u16)op2;
         Ai = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         sprintf( OPformat, "LDW @[A%u + %u], A%u", An, disp16, Ai );
         break;
      case 133:
         Dn = (u8)((op1 & (u16)0x0700) >> 8);
         imm6 = (u8)(op1 & (u16)0x003f);
         sprintf( OPformat, "CMPGE R%u, #%u", Dn, imm6 );
         break;
      case 134:
         An = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         imm5 = (u8)(op1 & (u16)0x001f);
         sprintf( OPformat, "ADD A%u, #%u", An, imm5 );
         break;
      case 135:
         An = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         imm5 = (u8)(op1 & (u16)0x001f);
         sprintf( OPformat, "SUB A%u, #%u", An, imm5 );
         break;
      case 136:
         An = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         imm22 = (u32)(((op1 & (u16)0x003f) << 16) | op2);
         sprintf( OPformat, "CMPEQ A%u, #%06XH", An, imm22 );
         break;
      case 137:
         An = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         imm22 = (u32)(((op1 & (u16)0x003f) << 16) | op2);
         sprintf( OPformat, "LD A%u, #%06XH", An, imm22 );
         break;
      case 138:
         An = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "ADD A%u, R%u", An, Ri );
         break;
      case 139:
         An = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         Ri = (u8)(op1 & (u16)0x000f);
         sprintf( OPformat, "SUB A%u, R%u", An, Ri );
         break;
      case 140:
         Dn = (u8)((op1 & (u16)0x0700) >> 8);
         Di = (u8)(op1 & (u16)0x0007);
         sprintf( OPformat, "MULUU R%u, R%u", Dn, Di );
         break;
      case 141:
         Dn = (u8)((op1 & (u16)0x0700) >> 8);
         Di = (u8)(op1 & (u16)0x0007);
         sprintf( OPformat, "MULUS R%u, R%u", Dn, Di );
         break;
      case 142:
         Dn = (u8)((op1 & (u16)0x0700) >> 8);
         Di = (u8)(op1 & (u16)0x0007);
         sprintf( OPformat, "MULSU R%u, R%u", Dn, Di );
         break;
      case 143:
         Dn = (u8)((op1 & (u16)0x0700) >> 8);
         Di = (u8)(op1 & (u16)0x0007);
         sprintf( OPformat, "MULSS R%u, R%u", Dn, Di );
         break;
      case 144:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Rm = (u8)(op1 & (u16)0x0007);
         if( Rn != Rm )
            sprintf( OPformat, "POP R%u, R%u", Rn, Rm );
         else
            sprintf( OPformat, "POP R%u", Rn );
         break;
      case 145:
         sprintf( OPformat, "Reserved" );
         break;
      case 146:
         An = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         Am = (u8)((op1 & (u16)0x0007) + 8);
         if( An != Am )
            sprintf( OPformat, "POP A%u, A%u", An, Am );
         else
            sprintf( OPformat, "POP A%u", An );
         break;
      case 147:
         Rn = (u8)((op1 & (u16)0x0f00) >> 8);
         Rm = (u8)(op1 & (u16)0x0007);
         if( Rn != Rm )
            sprintf( OPformat, "PUSH R%u, R%u", Rn, Rm );
         else
            sprintf( OPformat, "PUSH R%u", Rn );
         break;
      case 148:
         sprintf( OPformat, "Reserved" );
         break;
      case 149:
         An = (u8)(((op1 & (u16)0x0700) >> 8) + 8);
         Am = (u8)((op1 & (u16)0x0007) + 8);
         if( An != Am )
            sprintf( OPformat, "PUSH A%u, A%u", An, Am );
         else
            sprintf( OPformat, "PUSH A%u", An );
         break;
      case 150:
         eoffset13 = sext13to32( (u16)((op1 & (u16)0x0fff) << 1) );
         ipr = ip + eoffset13;
         sprintf( OPformat, "BSRD %d (%06XH)", eoffset13, ipr );
         break;
      case 151:
         EC = (u8)((op1 & (u16)0x0180) >> 7);
         eoffset8 = sext8to32( (u8)((op1 & (u16)0x007f) << 1) );
         ipr = ip + eoffset8;
         sprintf( OPformat, "BRA EC%u, %d (%06XH)", EC, eoffset8, ipr );
         break;
      case 152:
         sprintf( OPformat, "Reserved" );
         break;
      case 153:
         EC = (u8)((op1 & (u16)0x0180) >> 7);
         eoffset8 = sext8to32( (u8)((op1 & (u16)0x007f) << 1) );
         ipr = ip + eoffset8;
         sprintf( OPformat, "BRAD EC%u, %d (%06XH)", EC, eoffset8, ipr );
         break;
      case 154:
         H = (u8)(((op1 & (u16)0x0100) >> 9) + 6);
         eoffset8 = sext8to32( (u8)((op1 & (u16)0x007f) << 1) );
         ipr = ip + eoffset8;
         sprintf( OPformat, "BNZD R%u, %d (%06XH)", H, eoffset8, ipr );
         break;
      case 155:
         sprintf( OPformat, "Reserved" );
         break;
      case 156:
         eoffset11 = sext11to32( (u16)((op1 & (u16)0x03ff) << 1) );
         ipr = ip + eoffset11;
         sprintf( OPformat, "BRA %d (%06XH)", eoffset11, ipr );
         break;
      case 157:
         eoffset11 = sext11to32( (u16)((op1 & (u16)0x03ff) << 1) );
         ipr = ip + eoffset11;
         sprintf( OPformat, "BRAD %d (%06XH)", eoffset11, ipr );
         break;
      case 158:
         eoffset11 = sext11to32( (u16)((op1 & (u16)0x03ff) << 1) );
         ipr = ip + eoffset11;
         sprintf( OPformat, "BRF %d (%06XH)", eoffset11, ipr );
         break;
      case 159:
         eoffset11 = sext11to32( (u16)((op1 & (u16)0x03ff) << 1) );
         ipr = ip + eoffset11;
         sprintf( OPformat, "BRFD %d (%06XH)", eoffset11, ipr );
         break;
      case 160:
         eoffset11 = sext11to32( (u16)((op1 & (u16)0x03ff) << 1) );
         ipr = ip + eoffset11;
         sprintf( OPformat, "BRT %d (%06XH)", eoffset11, ipr );
         break;
      case 161:
         eoffset11 = sext11to32( (u16)((op1 & (u16)0x03ff) << 1) );
         ipr = ip + eoffset11;
         sprintf( OPformat, "BRTD %d (%06XH)", eoffset11, ipr );
         break;
      case 162:
         Dn = (u8)(op1 & (u16)0x0007);
         imm5 = (u8)((op1 & (u16)0x01f0) >> 4);
         sprintf( OPformat, "CLD R%u, %u", Dn, imm5 );
         break;
      case 163:
         imm5 = (u8)((op1 & (u16)0x01f0) >> 4);
         Di = (u8)(op1 & (u16)0x0007);
         sprintf( OPformat, "CLD %u, R%u", imm5, Di );
         break;
      default:
         if( mnnum >= 164 &&  mnnum <= 282 ) {
            imm13 = (u16)(op1 & (u16)0x1fff);
            sprintf( OPformat, "%s", formatCOP( mnnum - 164, imm13, op2 ) );
         }
         else
            sprintf( OPformat, "Invalid OP code" );
         break;
   }
   return OPformat;
}

